@@ -5,8 +5,8 @@
 ;; Author: Colin Walters <walters@verbum.org>
 ;; Maintainer: browse-kill-ring <browse-kill-ring@tonotdo.com>
 ;; Created: 7 Apr 2001
-;; Package-Version: 2.0.0
-;; Package-Revision: 2.0.0-0-g2a7acf98c348
+;; Package-Version: 20251208.1041
+;; Package-Revision: 26ea5759c996
 ;; URL: https://github.com/browse-kill-ring/browse-kill-ring
 ;; Keywords: convenience
 
@@ -50,8 +50,8 @@
 ;;; Code:
 
 (eval-when-compile
-  (require 'cl)
-  (require 'derived))
+  (require 'cl-lib))
+(require 'delsel)
 
 (defgroup browse-kill-ring nil
   "A package for browsing and inserting the items in `kill-ring'."
@@ -176,38 +176,6 @@ if non-nil, then display leftmost(last) duplicate items in `kill-ring'."
   :type 'boolean
   :group 'browse-kill-ring)
 
-(defadvice kill-new (around browse-kill-ring-no-kill-new-duplicates)
-  "An advice for not adding duplicate elements to `kill-ring'.
-Even after being \"activated\", this advice will only modify the
-behavior of `kill-new' when `browse-kill-ring-no-duplicates'
-is non-nil."
-  (if browse-kill-ring-no-duplicates
-      (setq kill-ring (delete (ad-get-arg 0) kill-ring)))
-  ad-do-it)
-
-(defcustom browse-kill-ring-no-duplicates nil
-  "If non-nil, then the `b-k-r-no-kill-new-duplicates' advice will operate.
-This means that duplicate entries won't be added to the `kill-ring'
-when you call `kill-new'.
-
-If you set this variable via customize, the advice will be activated
-or deactivated automatically.  Otherwise, to enable the advice, add
-
-B (ad-enable-advice 'kill-new 'around 'browse-kill-ring-no-kill-new-duplicates)
- (ad-activate 'kill-new)
-
-to your init file."
-  :type 'boolean
-  :set (lambda (symbol value)
-         (set symbol value)
-         (if value
-             (ad-enable-advice 'kill-new 'around
-                               'browse-kill-ring-no-kill-new-duplicates)
-           (ad-disable-advice 'kill-new 'around
-                              'browse-kill-ring-no-kill-new-duplicates))
-         (ad-activate 'kill-new))
-  :group 'browse-kill-ring)
-
 (defcustom browse-kill-ring-depropertize nil
   "If non-nil, remove text properties from `kill-ring' items.
 This only changes the items for display and insertion from
@@ -321,6 +289,31 @@ well."
                               (point)
                               quit))
 
+(defun browse-kill-ring-insert-new (insert-action post-action &optional quit)
+  "Insert the kill ring item at point into the last selected buffer.
+INSERT-ACTION can be \\='insert \\='append \\='prepend.
+POST-ACTION can be nil \\='move \\='delete.
+If optional argument QUIT is non-nil, close the *Kill Ring* buffer as
+well."
+  (interactive "P")
+  (let* ((buf (current-buffer))
+        (pt (point))
+        (str (browse-kill-ring-current-string buf pt)))
+    (cl-case insert-action
+      (insert (browse-kill-ring-do-insert buf pt nil))
+      (append (browse-kill-ring-do-append-insert buf pt nil))
+      (prepend (browse-kill-ring-do-prepend-insert buf pt nil))
+      (t (error "Unknown insert-action: %s" insert-action)))
+    (cl-case post-action
+      (move
+        (browse-kill-ring-delete)
+        (kill-new str))
+      (delete (browse-kill-ring-delete))
+      (t (error "Unknown post-action: %s" post-action)))
+    (if quit
+      (browse-kill-ring-quit)
+      (browse-kill-ring-update))))
+
 (defun browse-kill-ring-insert-and-delete (&optional quit)
   "Insert the kill ring item at point, and remove it from the kill ring.
 If optional argument QUIT is non-nil, close the *Kill Ring* buffer as
@@ -351,7 +344,7 @@ well."
 (defun browse-kill-ring-insert-move-and-quit ()
   "Like `browse-kill-ring-insert-and-move', but close the *Kill Ring* buffer."
   (interactive)
-  (browse-kill-ring-insert-and-move t))
+  (browse-kill-ring-insert-new 'insert 'move t))
 
 (defun browse-kill-ring-prepend-insert (&optional quit)
   "Like `browse-kill-ring-insert', but it places the entry at the beginning
@@ -395,12 +388,13 @@ of the *Kill Ring*."
         (warn "Unable to load `pulse' library")
         (setq browse-kill-ring-highlight-inserted-item 'solid)))
 
-    (case browse-kill-ring-highlight-inserted-item
+    (cl-case browse-kill-ring-highlight-inserted-item
       ((pulse t)
        (let ((pulse-delay .05) (pulse-iterations 10))
-         (pulse-momentary-highlight-region
-          start end browse-kill-ring-inserted-item-face)))
-      ('solid
+         (with-no-warnings
+           (pulse-momentary-highlight-region
+          start end browse-kill-ring-inserted-item-face))))
+      (solid
        (let ((o (make-overlay start end)))
          (overlay-put o 'face browse-kill-ring-inserted-item-face)
          (sit-for 0.5)
@@ -488,6 +482,9 @@ of the *Kill Ring*."
            (inhibit-read-only t))
       (delete-region (overlay-start over) (1+ (overlay-end over)))
       (setq kill-ring (delete target kill-ring))
+      (if (equal target (car kill-ring-yank-pointer))
+          (setq kill-ring-yank-pointer
+                (delete target kill-ring-yank-pointer)))
       (cond
        ;; Don't try to delete anything else in an empty buffer.
        ((and (bobp) (eobp)) t)
@@ -501,8 +498,8 @@ of the *Kill Ring*."
        ((get-text-property (point) 'browse-kill-ring-extra)
         (let ((prev (previous-single-property-change (point) 'browse-kill-ring-extra))
               (next (next-single-property-change (point) 'browse-kill-ring-extra)))
-          (when prev (incf prev))
-          (when next (incf next))
+          (when prev (cl-incf prev))
+          (when next (cl-incf next))
           (delete-region (or prev (point-min)) (or next (point-max))))))))
   (browse-kill-ring-resize-window)
   (browse-kill-ring-forward 0))
@@ -511,7 +508,7 @@ of the *Kill Ring*."
 (defun browse-kill-ring-target-overlay-at (position &optional no-error)
   "Return overlay at POSITION that has property `browse-kill-ring-target'.
 If no such overlay, raise an error unless NO-ERROR is true, in which
-case retun nil."
+case return nil."
   (let ((ovs  (overlays-at (point))))
     (catch 'browse-kill-ring-target-overlay-at
       (dolist (ov  ovs)
@@ -548,12 +545,12 @@ case retun nil."
 
 (defun browse-kill-ring-clear-highlighed-entry ()
   (when browse-kill-ring-previous-overlay
-    (assert (overlayp browse-kill-ring-previous-overlay))
+    (cl-assert (overlayp browse-kill-ring-previous-overlay))
     (overlay-put browse-kill-ring-previous-overlay 'face nil)))
 
 (defun browse-kill-ring-update-highlighed-entry-1 ()
   (let ((current-overlay (browse-kill-ring-target-overlay-at (point) t)))
-    (case current-overlay
+    (cl-case current-overlay
       ;; No overlay at point.  Just clear all current highlighting.
       ((nil) (browse-kill-ring-clear-highlighed-entry))
       ;; Still on the previous overlay.
@@ -561,7 +558,7 @@ case retun nil."
       ;; Otherwise, we've changed overlay.  Clear current
       ;; highlighting, and highlight the new overlay.
       (t
-       (assert (overlay-get current-overlay
+       (cl-assert (overlay-get current-overlay
                             'browse-kill-ring-target) t)
        (browse-kill-ring-clear-highlighed-entry)
        (setq browse-kill-ring-previous-overlay current-overlay)
@@ -576,7 +573,7 @@ case retun nil."
     (let ((o (browse-kill-ring-target-overlay-at (point) t)))
       (if (< arg 0)
           (progn
-            (incf arg)
+            (cl-incf arg)
             (when o
               (goto-char (overlay-start o))
               (setq o nil))
@@ -584,7 +581,7 @@ case retun nil."
               (goto-char (previous-overlay-change (point)))
               (setq o (browse-kill-ring-target-overlay-at (point) t))))
         (progn
-          (decf arg)
+          (cl-decf arg)
           ;; We're on a browse-kill-ring overlay, skip to the end of it.
           (when o
             (goto-char (overlay-end o))
@@ -600,11 +597,11 @@ case retun nil."
   (interactive "p")
   (browse-kill-ring-forward (- arg)))
 
-(defun browse-kill-ring-read-regexp (msg)
+(defun browse-kill-ring-read-regexp (msg &optional empty-is-nil-p)
   (let* ((default (car regexp-history))
          (input
           (read-from-minibuffer
-           (if default
+           (if (and default (not empty-is-nil-p))
                (format "%s for regexp (default `%s'): "
                        msg
                        default)
@@ -612,9 +609,10 @@ case retun nil."
            nil
            nil
            nil
-           'regexp-history)))
+           'regexp-history
+           (if empty-is-nil-p default nil))))
     (if (equal input "")
-        default
+        (if empty-is-nil-p nil default)
       input)))
 
 (defun browse-kill-ring-search-forward (regexp &optional backwards)
@@ -652,7 +650,7 @@ entry."
   "Take the action specified by `browse-kill-ring-quit-action'."
   (interactive)
   (browse-kill-ring-cleanup-on-exit)
-  (case browse-kill-ring-quit-action
+  (cl-case browse-kill-ring-quit-action
     (save-and-restore
       (if (< emacs-major-version 24)
         (let (buf (current-buffer))
@@ -678,10 +676,14 @@ You most likely do not want to call `browse-kill-ring-mode' directly; use
 `browse-kill-ring' instead.
 
 \\{browse-kill-ring-mode-map}"
+  ;; Later versions of emacs reduced the number of arguments to
+  ;; font-lock-defaults, at least version 24 requires 5 arguments
+  ;; before setting up buffer local variables.
   (set (make-local-variable 'font-lock-defaults)
        '(nil t nil nil nil
              (font-lock-fontify-region-function . browse-kill-ring-fontify-region)))
   (define-key browse-kill-ring-mode-map (kbd "q") 'browse-kill-ring-quit)
+  (define-key browse-kill-ring-mode-map (kbd "C-g") 'browse-kill-ring-quit)
   (define-key browse-kill-ring-mode-map (kbd "U") 'browse-kill-ring-undo-other-window)
   (define-key browse-kill-ring-mode-map (kbd "d") 'browse-kill-ring-delete)
   (define-key browse-kill-ring-mode-map (kbd "s") 'browse-kill-ring-search-forward)
@@ -736,7 +738,9 @@ directly; use `browse-kill-ring' instead.
   (define-key browse-kill-ring-edit-mode-map
     (kbd "C-c C-c") 'browse-kill-ring-edit-finish)
   (define-key browse-kill-ring-edit-mode-map
-    (kbd "C-c C-k") 'browse-kill-ring-edit-abort))
+    (kbd "C-c C-k") 'browse-kill-ring-edit-abort)
+  (define-key browse-kill-ring-edit-mode-map
+    (kbd "C-g") 'browse-kill-ring-edit-abort))
 
 (defvar browse-kill-ring-edit-target nil)
 (make-variable-buffer-local 'browse-kill-ring-edit-target)
@@ -766,42 +770,52 @@ update entry and quit -- \\[browse-kill-ring-edit-abort] to abort.")))
                 'browse-kill-ring-preview-update-for-edit nil t))
     (setq browse-kill-ring-edit-target target-cell)))
 
+(defun browse-kill-ring-edit-finalise (entry)
+  "Common code called after `browse-kill-ring-edit' has finished
+
+This common code is called after `browse-kill-ring-edit-finish'
+and `browse-kill-ring-edit-abort'.  It kills the edit buffer, and
+reselects ENTRY in the `*Kill Ring*' buffer."
+  ;; Kill the edit buffer.  Maybe we should do more to keep track of
+  ;; the edit buffer so we can kill it even if we're not in it?
+  (when (eq major-mode 'browse-kill-ring-edit-mode)
+    (kill-buffer))
+  ;; The user might have rearranged the windows
+  (when (eq major-mode 'browse-kill-ring-mode)
+    (browse-kill-ring-setup (current-buffer)
+                            browse-kill-ring-original-buffer
+                            browse-kill-ring-original-window
+                            nil
+                            browse-kill-ring-original-window-config)
+    (browse-kill-ring-resize-window)
+    (when entry
+      (browse-kill-ring-find-entry entry))))
+
 (defun browse-kill-ring-edit-finish ()
-  "Commit the changes to the `kill-ring'."
+  "Commit the edit changes to the `kill-ring'."
   (interactive)
+  (unless browse-kill-ring-edit-target
+    (error "Not editing a kill-ring item"))
   (let* ((updated-entry (buffer-string))
          (delete-entry? (string= updated-entry ""))
+         (current-entry browse-kill-ring-edit-target)
          (select-entry nil))
-    (if browse-kill-ring-edit-target
-        (if delete-entry?
-            ;; Find the previous entry in the list to select, then
-            ;; delete the entry that was just edited to empty.
-            (progn
-              (setq select-entry
-                    (cadr browse-kill-ring-edit-target))
-              (setq kill-ring
-                    (delete (car browse-kill-ring-edit-target) kill-ring))
-              (unless select-entry
-                (setq select-entry (car (last kill-ring)))))
-          ;; Update the entry that was just edited, and arrange to
-          ;; select it.
-          (setcar browse-kill-ring-edit-target updated-entry)
-          (setq select-entry updated-entry))
-      (unless delete-entry?
-        (when (y-or-n-p "The item has been deleted; add to front? ")
-          (push updated-entry kill-ring)
-          (setq select-entry updated-entry))))
-    (kill-buffer)
-    ;; The user might have rearranged the windows
-    (when (eq major-mode 'browse-kill-ring-mode)
-      (browse-kill-ring-setup (current-buffer)
-                              browse-kill-ring-original-buffer
-                              browse-kill-ring-original-window
-                              nil
-                              browse-kill-ring-original-window-config)
-      (browse-kill-ring-resize-window)
-      (when select-entry
-        (browse-kill-ring-find-entry select-entry)))))
+    (setq browse-kill-ring-edit-target nil)
+    (if delete-entry?
+        ;; Find the previous entry in the list to select, then
+        ;; delete the entry that was just edited to empty.
+        (progn
+          (setq select-entry
+                (cadr current-entry))
+          (setq kill-ring
+                (delete (car current-entry) kill-ring))
+          (unless select-entry
+            (setq select-entry (car (last kill-ring)))))
+      ;; Update the entry that was just edited, and arrange to select
+      ;; it.
+      (setcar current-entry updated-entry)
+      (setq select-entry updated-entry))
+    (browse-kill-ring-edit-finalise select-entry)))
 
 (defun browse-kill-ring-edit-abort ()
   "Abort the edit of the `kill-ring' item."
@@ -809,21 +823,12 @@ update entry and quit -- \\[browse-kill-ring-edit-abort] to abort.")))
   (let ((current-entry (if browse-kill-ring-edit-target
                            (car browse-kill-ring-edit-target)
                          nil)))
-    (kill-buffer)
-    ;; The user might have rearranged the windows
-    (when (eq major-mode 'browse-kill-ring-mode)
-      (browse-kill-ring-setup (current-buffer)
-                              browse-kill-ring-original-buffer
-                              browse-kill-ring-original-window
-                              nil
-                              browse-kill-ring-original-window-config)
-      (browse-kill-ring-resize-window))
-    (if current-entry
-        (browse-kill-ring-find-entry current-entry))))
+    (setq browse-kill-ring-edit-target nil)
+    (browse-kill-ring-edit-finalise current-entry)))
 
 (defmacro browse-kill-ring-add-overlays-for (item &rest body)
-  (let ((beg (gensym "browse-kill-ring-add-overlays-"))
-        (end (gensym "browse-kill-ring-add-overlays-")))
+  (let ((beg (cl-gensym "browse-kill-ring-add-overlays-"))
+        (end (cl-gensym "browse-kill-ring-add-overlays-")))
     `(let ((,beg (point))
            (,end
             (progn
@@ -885,9 +890,9 @@ update entry and quit -- \\[browse-kill-ring-edit-abort] to abort.")))
 (defun browse-kill-ring-occur (regexp)
   "Display all `kill-ring' entries matching REGEXP."
   (interactive
-   (list
-    (browse-kill-ring-read-regexp "Display kill ring entries matching")))
-  (assert (eq major-mode 'browse-kill-ring-mode))
+   (list (browse-kill-ring-read-regexp
+          "Display kill ring entries matching" t)))
+  (cl-assert (eq major-mode 'browse-kill-ring-mode))
   (browse-kill-ring-setup (current-buffer)
                           browse-kill-ring-original-buffer
                           browse-kill-ring-original-window
@@ -914,13 +919,14 @@ update entry and quit -- \\[browse-kill-ring-edit-abort] to abort.")))
   (let ((buffer-read-only nil))
     (browse-kill-ring-fontify-on-property 'browse-kill-ring-extra 'bold beg end)
     (browse-kill-ring-fontify-on-property 'browse-kill-ring-separator
-                                          browse-kill-ring-separator-face beg end))
+                                          browse-kill-ring-separator-face beg end)
+    (font-lock-fontify-keywords-region beg end verbose))
   (when verbose (message "Fontifying...done")))
 
 (defun browse-kill-ring-update ()
   "Update the buffer to reflect outside changes to `kill-ring'."
   (interactive)
-  (assert (eq major-mode 'browse-kill-ring-mode))
+  (cl-assert (eq major-mode 'browse-kill-ring-mode))
   (browse-kill-ring-setup (current-buffer)
                           browse-kill-ring-original-buffer
                           browse-kill-ring-original-window)
@@ -929,7 +935,7 @@ update entry and quit -- \\[browse-kill-ring-edit-abort] to abort.")))
 (defun browse-kill-ring-preview-update-text (preview-text)
   "Update `browse-kill-ring-preview-overlay' to show `PREVIEW-TEXT`."
   ;; If preview-text is nil, replacement should be nil too.
-  (assert (overlayp browse-kill-ring-preview-overlay))
+  (cl-assert (overlayp browse-kill-ring-preview-overlay))
   (let ((replacement (when preview-text
                        (propertize preview-text 'face 'highlight))))
     (overlay-put browse-kill-ring-preview-overlay
@@ -939,7 +945,7 @@ update entry and quit -- \\[browse-kill-ring-edit-abort] to abort.")))
   "Update `browse-kill-ring-preview-overlay' to match item at PT.
 This function is called whenever the selection in the `*Kill
 Ring*' buffer is adjusted, the `browse-kill-ring-preview-overlay'
-is udpated to preview the text of the selection at PT (or the
+is updated to preview the text of the selection at PT (or the
 current point if not specified)."
   (let ((new-text (browse-kill-ring-current-string
                    (current-buffer) (or pt (point)) t)))
@@ -1064,8 +1070,7 @@ it's turned on."
               ;; display leftmost or rightmost duplicate.
               ;; if `browse-kill-ring-display-leftmost-duplicate' is t,
               ;; display leftmost(last) duplicate.
-              (require 'cl)
-              (delete-duplicates items
+              (cl-delete-duplicates items
                                  :test #'equal
                                  :from-end browse-kill-ring-display-leftmost-duplicate))
             (when (stringp regexp)
@@ -1111,8 +1116,9 @@ it's turned on."
             (set-buffer-modified-p nil)
             (goto-char (point-min))
             (browse-kill-ring-forward 0)
-            (when regexp
-              (setq mode-name (concat "Kill Ring [" regexp "]")))
+            (setq mode-name (if regexp
+                                (concat "Kill Ring [" regexp "]")
+                              "Kill Ring"))
             (run-hooks 'browse-kill-ring-hook)))
       (progn
         (setq buffer-read-only t)))))
@@ -1146,18 +1152,19 @@ start of the buffer."
   "Display items in the `kill-ring' in another buffer."
   (interactive)
   (if (eq major-mode 'browse-kill-ring-mode)
-      (message "Already viewing the kill ring")
-    (let* ((orig-win (selected-window))
-           (orig-buf (window-buffer orig-win))
-           (buf (get-buffer-create "*Kill Ring*"))
-           (kill-ring-yank-pointer-string
-            (if kill-ring-yank-pointer
-                (substring-no-properties (car kill-ring-yank-pointer)))))
-      (browse-kill-ring-setup buf orig-buf orig-win)
-      (pop-to-buffer buf)
-      (browse-kill-ring-resize-window)
-      (unless (eq kill-ring kill-ring-yank-pointer)
-        (browse-kill-ring-find-entry kill-ring-yank-pointer-string)))))
+      (error "Already viewing the kill ring"))
+
+  (let* ((orig-win (selected-window))
+         (orig-buf (window-buffer orig-win))
+         (buf (get-buffer-create "*Kill Ring*"))
+         (kill-ring-yank-pointer-string
+          (if kill-ring-yank-pointer
+              (substring-no-properties (car kill-ring-yank-pointer)))))
+    (browse-kill-ring-setup buf orig-buf orig-win)
+    (pop-to-buffer buf)
+    (browse-kill-ring-resize-window)
+    (unless (eq kill-ring kill-ring-yank-pointer)
+      (browse-kill-ring-find-entry kill-ring-yank-pointer-string))))
 
 (provide 'browse-kill-ring)
 
